%TEST_SGCEN Test program for SGCEN package.
%   This script identifies the most important nodes of two test networks,
%   according to subgraph centrality, using four different methods.

%   Caterina Fenu & Giuseppe Rodriguez, University of Cagliari, Italy
%   David Martin & Lothar Reichel, Kent State University, U.S.A.
%   Email: kate.fenu@gmail.com, rodriguez@unica.it, 
%          dmarti49@kent.edu, reichel@math.kent.edu
%
%   Last revised August 2, 2013

% test matrices
testmats = {
	'yeast'
	'power'
	};
nmatr = size(testmats,1);

clc
fprintf('\nComputation of subgraph centrality.\n\n')
for i = 1:nmatr
	fprintf('%3d - %s\n', i, testmats{i})
end
fprintf('\nPress enter for default values.\n')
tst = input('choose a test matrix (def. 1): ');
if isempty(tst), tst = 1; end
load(sprintf('adjacency_%s',testmats{tst}))
n = size(A,1);
fprintf('number of nodes: %d\n',n)
numnod = input('choose the number of nodes to be identified (def. 5): ');
if isempty(numnod), numnod = 5; end

matrixname = testmats{tst};

% options
clear opts
opts.gausstolq = 1e-3;		% quadrature accuracy
opts.sgctoll = 1e-3;		% low-rank accuracy
opts.sgcshift = 1;		% spectrum shift
opts.gaussreorth = 0;		% reorthogonalization

fprintf('Computing subgraph centrality by expm ')
tic
allx = diag(expm(A));
[y i] = sort(allx,'descend');
sc = y(1:numnod);
x = i(1:numnod);
fprintf('(%.2f seconds)\n',toc)

fprintf('Computing subgraph centrality by Gauss-Lanczos ')
tic
[vipg sgg infog] = sgcengauss(A,numnod,opts);
fprintf('(%.2f seconds)\n',toc)

fprintf('Computing subgraph centrality by low-rank approx. (weak conv.) ')
tic
[viplr sglr infolr] = sgcenlr(A,numnod,opts);
viplr = viplr(1:numnod);
sglr = sglr(1:numnod);
fprintf('(%.2f seconds)\n',toc)

fprintf('Computing subgraph centrality by hybrid algorithm ')
tic
[viph sgh infoh] = sgcenhyb(A,numnod,opts,[]);
fprintf('(%.2f seconds)\n',toc)

results = [ (1:numnod)' x sc vipg sgg viplr sglr viph sgh ];
fprintf('\n          EXPM           GAUSS          LOW-RANK         HYBRID \n')
fprintf('%3d.%5d(%.2e) %5d(%.2e) %5d(%.2e) %5d(%.2e)\n', results')
fprintf('\nThe table displays the indexes of the nodes with the largest\n')
fprintf('subgraph centrality, which is reported in parentheses.\n\n')

